package provide DS9 1.0

# Public Functions

proc CreateColorbar {} {
    global colorbar
    global ds9

    set colorbar(name) \
	[canvas .ds9.cmap -width $colorbar(width) -height $colorbar(height) \
	     -bd 2 -relief groove -insertofftime 0]
    switch -- $ds9(visual) {
	pseudocolor {
	    $colorbar(name) create colorbar$ds9(visual)$ds9(depth) \
		-private $colorbar(private) \
		-width $colorbar(width) -height $colorbar(height) -anchor nw
	}
	truecolor {
	    $colorbar(name) create colorbar$ds9(visual)$ds9(depth) \
		-width $colorbar(width) -height $colorbar(height) -anchor nw -colors 2048
	}
    }

    # And set the two windows that where created before the colorbar was created,
    # to the new colormap

    colorbar set colormap window .ds9

    # This is a kluge, but it works. We want to reassign the bg color for .ds9
    # from the old colormap to the new colormap

    .ds9 configure -background [.ds9 cget -background]
}

proc InitColorbar {} {
    global colorbar

    colorbar map $colorbar(map)
    colorbar invert $colorbar(invert)
    bind $colorbar(name) <Configure> [list UpdateColorbarGeometry]
}

proc LoadColorMap {} {
    global ds9
    global colorbar

    set fileName [OpenFileDialog $ds9(top) ds9(lastColormapName)]
    if {$fileName != ""} {
	colorbar load $fileName
	set id [colorbar get id]
	set colorbar(map) [colorbar get name]
	.menuBar.color insert $colorbar(count) radiobutton -label "$colorbar(map)" \
	    -variable colorbar(map) -command "ChangeColormapID $id"
	incr colorbar(count)
    }
}

proc SaveColorMap {} {
    global ds9

    set init [colorbar get file name]
    set fileName [SaveFileDialog $ds9(top) init]
    if {$fileName != ""} {
	colorbar save $fileName
    }
}

proc ButtonColormap {x y} {
    global current

    if {$current(frame) != ""} {
	$current(frame) colormap begin
    }
}

proc MotionColormap {x y} {
    global canvas
    global cursor
    global current

    set cursorX [expr double($x-$canvas(width)/2+$cursor(speed)/2)/$cursor(speed)]
    set cursorY [expr double($y)/$cursor(speed) * $cursor(mult)]

    colorbar adjust $cursorX $cursorY
    if {$current(frame) != ""} {
	$current(frame) colormap motion [colorbar get colormap]
    }
}

proc ReleaseColormap {x y} {
    global current

    if {$current(frame) != ""} {
	$current(frame) colormap end [colorbar get colormap]
    }
}

proc CreateColorMenu {} {
    global colorbar

    set id [colorbar list id]

    foreach i $id {
	set name [colorbar get name $i]
	.menuBar.color insert $colorbar(count) radiobutton -label "$name" \
	    -variable colorbar(map) -command "ChangeColormapID $i"
	incr colorbar(count)
    }
}

proc UpdateColorbarGeometry {} {
    global colorbar

    set colorbar(width) [expr [winfo width $colorbar(name)]-2]
    set colorbar(height) [expr [winfo height $colorbar(name)]-2]

    colorbar configure -width $colorbar(width) -height $colorbar(height)
}

proc ChangeColormapID {id} {
    global current
    global colorbar
    
    SetWatchCursor
    colorbar map $id
    if {$current(frame) != "" } {
	$current(frame) colormap [colorbar get colormap]
	set colorbar(map) [colorbar get name]
	set colorbar(invert) [colorbar get invert]
    }
    UnsetWatchCursor
}

proc ChangeColormap {} {
    global current
    global colorbar
    
    SetWatchCursor
    colorbar map $colorbar(map)
    if {$current(frame) != "" } {
	$current(frame) colormap [colorbar get colormap]
	set colorbar(invert) [colorbar get invert]
    }
    UnsetWatchCursor
}

proc MatchColorbars {} {
    global ds9
    global current
    global colorbar

    SetWatchCursor
    foreach f $ds9(frames) {
	if {$f != $current(frame)} {
	    $f colormap [colorbar get colormap]
	}
    }
    UnsetWatchCursor
}

proc InvertColorbar {} {
    global colorbar
    global current

    SetWatchCursor
    colorbar invert $colorbar(invert)
    if {$current(frame) != "" } {
	$current(frame) colormap [colorbar get colormap]
    }
    UnsetWatchCursor
}
