package provide DS9 1.0

proc CreateInfoPanel {} {
    global ds9
    global pan
    global mag
    global panel
    global info

    # Panel Frame

    set panel(name) [frame .ds9.panel]

    # Panner

    set pan(name) [canvas $panel(name).pan -width $pan(size) \
		       -height $pan(size) \
		       -bd 2 -relief groove \
		       -highlightthickness 0 -insertofftime 0 -takefocus 0]
    $pan(name) create panner$ds9(visual) -width $pan(size) -height $pan(size) \
	-command panner

    # Magnifier

    set mag(name) [canvas $panel(name).mag -width $mag(size) \
		       -height $mag(size) \
		       -bd 2 -relief groove \
		       -highlightthickness 0 -insertofftime 0 -takefocus 0]
    $mag(name) create magnifier$ds9(visual) -width $mag(size) \
	-height $mag(size) \
	-command magnifier

    # Info

    set info(name) [frame $panel(name).info -relief groove -borderwidth 2]

    grid rowconfigure $info(name) 0 -pad 2
    grid rowconfigure $info(name) 1 -pad 2
    grid rowconfigure $info(name) 2 -pad 2
    grid rowconfigure $info(name) 3 -pad 2
    grid rowconfigure $info(name) 4 -pad 2
    grid rowconfigure $info(name) 5 -pad 2
    grid rowconfigure $info(name) 6 -pad 2
    grid rowconfigure $info(name) 7 -pad 2
    grid rowconfigure $info(name) 8 -pad 2
    grid rowconfigure $info(name) 9 -pad 2
    grid rowconfigure $info(name) 10 -pad 2

    label $info(name).fileTitle -text "File"
    label $info(name).fileValue -relief groove -bd 2 -anchor w -width 31 \
	-textvariable info(pFileName)

    label $info(name).objTitle -text "Object"
    label $info(name).objValue -relief groove -bd 2 -anchor w -width 31 \
	-textvariable info(pObject)

    label $info(name).minTitle -text "Min"
    label $info(name).minValue -relief groove -bd 2 -width 13 \
	-textvariable info(pMin)


    label $info(name).maxTitle -text "Max"
    label $info(name).maxValue -relief groove -bd 2 -width 13 \
	-textvariable info(pMax)

    label $info(name).lowTitle -text "Low"
    label $info(name).lowValue -relief groove -bd 2 -width 13 \
	-textvariable info(pLow)

    label $info(name).highTitle -text "High"
    label $info(name).highValue -relief groove -bd 2 -width 13 \
	-textvariable info(pHigh)

    label $info(name).valueTitle -text "Value" -width 9 -anchor w
    label $info(name).valueValue -relief groove -bd 2 -width 13 \
	-textvariable info(pValue)

    set info(pWCSCoord) "WCS"
    set info(pWCSXName) [label $info(name).wcsXLabel -text "" \
			     -font symbol -width 3]
    label $info(name).wcsXValue -relief groove -bd 2 -width 13 \
	-textvariable info(pWCSX)
    set info(pWCSYName) [label $info(name).wcsYLabel -text "" \
			     -font symbol -width 3]
    label $info(name).wcsYValue -relief groove -bd 2 -width 13 \
	-textvariable info(pWCSY)
    label $info(name).wcsCoord -anchor w -width 8 -textvariable info(pWCSCoord)

    label $info(name).detectorTitle -text "Detector"
    label $info(name).detectorXLabel -text "X"
    label $info(name).detectorXValue -relief groove -bd 2 -width 13 \
	-textvariable info(pDetectorX)
    label $info(name).detectorYLabel -text "Y"
    label $info(name).detectorYValue -relief groove -bd 2 -width 13 \
	-textvariable info(pDetectorY)

    label $info(name).amplifierTitle -text "Amp"
    label $info(name).amplifierXLabel -text "X"
    label $info(name).amplifierXValue -relief groove -bd 2 -width 13 \
	-textvariable info(pAmplifierX)
    label $info(name).amplifierYLabel -text "Y"
    label $info(name).amplifierYValue -relief groove -bd 2 -width 13 \
	-textvariable info(pAmplifierY)

    label $info(name).physicalTitle -text "Physical"
    label $info(name).physicalXLabel -text "X"
    label $info(name).physicalXValue -relief groove -bd 2 -width 13 \
	-textvariable info(pPhysicalX)
    label $info(name).physicalYLabel -text "Y"
    label $info(name).physicalYValue -relief groove -bd 2 -width 13 \
	-textvariable info(pPhysicalY)

    label $info(name).imageTitle -text "Image"
    label $info(name).imageXLabel -text "X"
    label $info(name).imageXValue -relief groove -bd 2 -width 13 \
	-textvariable info(pImageX)
    label $info(name).imageYLabel -text "Y"
    label $info(name).imageYValue -relief groove -bd 2 -width 13 \
	-textvariable info(pImageY)

    label $info(name).frame -textvariable info(pFrame)

    label $info(name).zoomTitle -text "Zoom"
    label $info(name).zoomValue -relief groove -bd 2 -width 13 \
	-textvariable info(pZoom)

    label $info(name).angleTitle -text "Ang"
    label $info(name).angleValue -relief groove -bd 2 -width 13 \
	-textvariable info(pAngle)

    set info(pFrame) Frame
}

proc UpdateInfoPanel {} {
    global info

    if {$info(filename)} {
	grid $info(name).fileTitle -row 0 -column 0 -sticky w
	grid $info(name).fileValue -row 0 -column 2 -padx 2 \
	    -sticky w -columnspan 3
    } else {
	grid forget $info(name).fileTitle
	grid forget $info(name).fileValue
    }

    if {$info(object)} {
	grid $info(name).objTitle -row 1 -column 0 -sticky w
	grid $info(name).objValue -row 1 -column 2 -padx 2 \
	    -sticky w -columnspan 3
    } else {
	grid forget $info(name).objTitle
	grid forget $info(name).objValue
    }

    if {$info(minmax)} {
	grid $info(name).minTitle -row 2 -column 0 -sticky w
	grid $info(name).minValue -row 2 -column 2 -padx 2 -sticky w
	grid $info(name).maxTitle -row 2 -column 3 -sticky w
	grid $info(name).maxValue -row 2 -column 4 -padx 2 -sticky w
	grid $info(name).lowTitle -row 3 -column 0 -sticky w
	grid $info(name).lowValue -row 3 -column 2 -padx 2 -sticky w
	grid $info(name).highTitle -row 3 -column 3 -sticky w
	grid $info(name).highValue -row 3 -column 4 -padx 2 -sticky w
    } else {
	grid forget $info(name).minTitle
	grid forget $info(name).minValue
	grid forget $info(name).maxTitle
	grid forget $info(name).maxValue
	grid forget $info(name).lowTitle
	grid forget $info(name).lowValue
	grid forget $info(name).highTitle
	grid forget $info(name).highValue
    }

    grid $info(name).valueTitle -row 4 -column 0 -sticky w
    grid $info(name).valueValue -row 4 -column 2 -padx 2

    if {$info(wcs)} {
	grid $info(name).wcsCoord -row 5 -column 0 -sticky w
	grid $info(name).wcsXLabel -row 5 -column 1 -sticky e
	grid $info(name).wcsXValue -row 5 -column 2 -padx 2
	grid $info(name).wcsYLabel -row 5 -column 3 -sticky e
	grid $info(name).wcsYValue -row 5 -column 4 -padx 2
    } else {
	grid forget $info(name).wcsCoord
	grid forget $info(name).wcsXLabel
	grid forget $info(name).wcsXValue
	grid forget $info(name).wcsYLabel
	grid forget $info(name).wcsYValue
    }
    
    if {$info(detector)} {
	grid $info(name).detectorTitle -row 6 -column 0 -sticky w
	grid $info(name).detectorXLabel -row 6 -column 1 -sticky e
	grid $info(name).detectorXValue -row 6 -column 2 -padx 2
	grid $info(name).detectorYLabel -row 6 -column 3 -sticky e
	grid $info(name).detectorYValue -row 6 -column 4 -padx 2
    } else {
	grid forget $info(name).detectorTitle
	grid forget $info(name).detectorXLabel
	grid forget $info(name).detectorXValue
	grid forget $info(name).detectorYLabel
	grid forget $info(name).detectorYValue
    }

    if {$info(amplifier)} {
	grid $info(name).amplifierTitle -row 7 -column 0 -sticky w
	grid $info(name).amplifierXLabel -row 7 -column 1 -sticky e
	grid $info(name).amplifierXValue -row 7 -column 2 -padx 2
	grid $info(name).amplifierYLabel -row 7 -column 3 -sticky e
	grid $info(name).amplifierYValue -row 7 -column 4 -padx 2
    } else {
	grid forget $info(name).amplifierTitle
	grid forget $info(name).amplifierXLabel
	grid forget $info(name).amplifierXValue
	grid forget $info(name).amplifierYLabel
	grid forget $info(name).amplifierYValue
    }

    if {$info(physical)} {
	grid $info(name).physicalTitle -row 8 -column 0 -sticky w
	grid $info(name).physicalXLabel -row 8 -column 1 -sticky e
	grid $info(name).physicalXValue -row 8 -column 2 -padx 2
	grid $info(name).physicalYLabel -row 8 -column 3 -sticky e
	grid $info(name).physicalYValue -row 8 -column 4 -padx 2
    } else {
	grid forget $info(name).physicalTitle
	grid forget $info(name).physicalXLabel
	grid forget $info(name).physicalXValue
	grid forget $info(name).physicalYLabel
	grid forget $info(name).physicalYValue
    }

    if {$info(image)} {
	grid $info(name).imageTitle -row 9 -column 0 -sticky w
	grid $info(name).imageXLabel -row 9 -column 1 -sticky e
	grid $info(name).imageXValue -row 9 -column 2 -padx 2
	grid $info(name).imageYLabel -row 9 -column 3 -sticky e
	grid $info(name).imageYValue -row 9 -column 4 -padx 2
    } else {
	grid forget $info(name).imageTitle
	grid forget $info(name).imageXLabel
	grid forget $info(name).imageXValue
	grid forget $info(name).imageYLabel
	grid forget $info(name).imageYValue
    }

    if {$info(frame)} {
	grid $info(name).frame -row 10 -column 0 -sticky w
	grid $info(name).zoomTitle -row 10 -column 1 -sticky w
	grid $info(name).zoomValue -row 10 -column 2 -padx 2 -sticky w
	grid $info(name).angleTitle -row 10 -column 3
	grid $info(name).angleValue -row 10 -column 4 -padx 2 -sticky w
    } else {
	grid forget $info(name).frame
	grid forget $info(name).zoomTitle
	grid forget $info(name).zoomValue
	grid forget $info(name).angleTitle
	grid forget $info(name).angleValue
    }
}

proc UpdateInfoBox {which x y coord} {
    global info
    global wcs

# Note: we want to avoid changing the displayed data at all cost. 
# Updating a label widget is very costly. Therefor, try to test 
# for change of data before updating.

    if {[$which get fits count] > 0} {

	$which get info $coord $x $y $wcs(system) $wcs(format) result

	if {$info(pFileName) != $result(fileName)} {
	    set info(pFileName) $result(fileName)
	}
	if {$info(pObject) != $result(objectName)} {
	    set info(pObject) $result(objectName)
	}

	if {$info(pMin) != $result(min)} {
	    set info(pMin) $result(min)
	}
	if {$info(pMax) != $result(max)} {
	    set info(pMax) $result(max)
	}
	if {$info(pLow) != $result(low)} {
	    set info(pLow) $result(low)
	}
	if {$info(pHigh) != $result(high)} {
	    set info(pHigh) $result(high)
	}

	set info(pValue) $result(value)

	if {$info(image)} {
	    set info(pImageX) $result(xImage)
	    set info(pImageY) $result(yImage)
	}

	if {$info(physical)} {
	    set info(pPhysicalX) $result(xPhysical)
	    set info(pPhysicalY) $result(yPhysical)
	}

	if {$info(amplifier)} {
	    set info(pAmplifierX) $result(xAmplifier)
	    set info(pAmplifierY) $result(yAmplifier)
	}

	if {$info(detector)} {
	    set info(pDetectorX) $result(xDetector)
	    set info(pDetectorY) $result(yDetector)
	}

	if {$info(wcs)} {
	    set info(pWCSX) $result(xWCS)
	    set info(pWCSY) $result(yWCS)

	    if {$result(sysWCS) == "fk4" \
		    || $result(sysWCS) == "fk5" \
		    || $result(sysWCS) == "icrs"} {
		set info(pWCSCoord) [string toupper $result(sysWCS)]
		$info(pWCSXName) configure -text "a" -font symbol
		$info(pWCSYName) configure -text "d" -font symbol
	    } elseif {$result(sysWCS) == "galactic"} {
		set info(pWCSCoord) "Galactic"
		$info(pWCSXName) configure -text "l" -font \
		    {-family times -slant italic}
		$info(pWCSYName) configure -text "b" -font \
		    {-family times -slant italic}
	    } elseif {$result(sysWCS) == "ecliptic"} {
		set info(pWCSCoord) "Ecliptic"
		$info(pWCSXName) configure -text "l" -font symbol
		$info(pWCSYName) configure -text "b" -font symbol
	    } elseif {$result(sysWCS) == "alt-az"} {
		set info(pWCSCoord) [string toupper $result(sysWCS)]
		$info(pWCSXName) configure -text "A" -font times
		$info(pWCSYName) configure -text "a" -font times
	    } elseif {$result(sysWCS) == "linear"} {
		set info(pWCSCoord) [string toupper $result(sysWCS)]
		set unit1 [string range $result(xUnitWCS) 0 2]
		set unit2 [string range $result(yUnitWCS) 0 2]
		$info(pWCSXName) configure -text "$unit1" -font times
		$info(pWCSYName) configure -text "$unit2" -font times
	    } else {
		set info(pWCSCoord) "WCS"
		$info(pWCSXName) configure -text ""
		$info(pWCSYName) configure -text ""
	    }
	}
    }
}

proc UpdateInfoFileName {which} {
    global info

    if {[$which get fits count] > 0} {
	set fn [$which get fits file name number 1]
	if {$info(pFileName) != $fn} {
	    set info(pFileName) $fn
	}

	set obj [$which get fits object name number 1]
	if {$info(pObject) != $obj} {
	    set info(pObject) $obj
	}
    }
}

proc ClearInfoBox {} {
    global info
    global current
    global ds9

    switch $ds9(displaymode) {
	single {
	    if {$current(frame) != ""} {
		if {[$current(frame) get fits count] > 1} {
		    # only clear for mosaics
		    set info(pFileName) ""
		    set info(pObject) ""

		    set info(pMin) ""
		    set info(pMax) ""
		    set info(pLow) ""
		    set info(pHigh) ""
		}
	    }
	}
	tile {
	    set info(pFileName) ""
	    set info(pObject) ""

	    set info(pMin) ""
	    set info(pMax) ""
	    set info(pLow) ""
	    set info(pHigh) ""
	}
	blink {}
    }

    set info(pValue) ""

    $info(pWCSXName) configure -text ""
    $info(pWCSYName) configure -text ""
    set info(pWCSX) ""
    set info(pWCSY) ""
    set info(pWCSCoord) "WCS"
    set info(pDetectorX) ""
    set info(pDetectorY) ""
    set info(pAmplifierX) ""
    set info(pAmplifierY) ""
    set info(pPhysicalX) ""
    set info(pPhysicalY) ""
    set info(pImageX) ""
    set info(pImageY) ""
}

proc GetCoords {which x y} {
    global ds9
    global coord

    if {$ds9(centerpixel)} {
	set x [expr $x+.5]
	set y [expr $y+.5]
    }

    set r ""

    if {[$which has wcs equatorial]} {
	if {$coord(fk4)} {
	    set r "$r [$which get wcs coordinates canvas $x $y fk4 $coord(format)]"
	}

	if {$coord(fk5)} {
	    set r "$r [$which get wcs coordinates canvas $x $y fk5 $coord(format)]"
	}

	if {$coord(icrs)} {
	    set r "$r [$which get wcs coordinates canvas $x $y icrs $coord(format)]"
	}

	if {$coord(galactic)} {
	    set r "$r [$which get wcs coordinates canvas $x $y galactic $coord(format)]"
	}
	
	if {$coord(ecliptic)} {
	    set r "$r [$which get wcs coordinates canvas $x $y ecliptic $coord(format)]"
	}
    }

    if {$coord(detector)} {
	if {[$which has mosaic] && [$which has physical]} {
	    set r "$r [$which get detector coordinates canvas $x $y] detector"
	}
    }

    if {$coord(amplifier)} {
	if {[$which has mosaic]} {
	    set r "$r [$which get amplifier coordinates canvas $x $y] amplifier"
	}
    }

    if {$coord(physical)} {
	if {[$which has physical]} {
	    set r "$r [$which get physical coordinates canvas $x $y] physical"
	}
    }

    if {$coord(image)} {
	set r "$r [$which get image coordinates canvas $x $y]"
    }

    if {$coord(value)} {
	set r "$r [$which get value canvas $x $y]"
    }

    set r "$r \n"

    return $r
}

proc DisplayCoordDialog {which x y} {
    global coordDialog

    set coordDialog(top) .coord
    set coordDialog(mb) .coordMB
    set coordDialog(txt) [GetCoords $which $x $y]

    SimpleTextDialog coordDialog 80 20 "Coordinates" append
}
