#include <string.h>

#include <iostream.h>
#include <strstream.h>

#include <tcl.h>

#include "iistcl.h"
#include "iis.h"

extern "C" {

void get_fbconfig(XimDataPtr);
void GtQueryColormap(void* w, int a, int* first, int* nelem, int* maxelem);
void GtReadPixels(void* gt, int frame, void* pixels, int nbits, 
		  int x, int y, int dx, int dy);
void GtWritePixels(void* gt, int frame, void* pixels, int bits,
		   int x, int y, int dx, int dy);

void xim_cursorMode(XimDataPtr xim, int state);
void xim_getCursorPos(XimDataPtr xim, int* x, int* y, int* raster, int* frame);
void xim_setReferenceFrame(IoChanPtr chan, int frame);
void xim_setDisplayFrame(XimDataPtr xim, int frame);
void xim_removeInput(XimDataPtr xim, int fd);
void xim_message(XimDataPtr xim, char* message, char* imtitle);
void xim_initFrame(XimDataPtr xim, int frame, int nframes, FbConfig* config,
		   char* memModel);
void xim_setCursorPos(XimDataPtr xim, int x, int y);
void xim_eraseFrame(XimDataPtr xim, int frame);
XtPointer xim_addInput(XimDataPtr xim, int fd, 
		       void (*func)(IoChan*, int*, void*), IoChan* chan);

void iisIO(void* foo, int mask)
{
  if (IISDEBUG)
    cerr << "iisIO() " << foo << ' ' << mask << endl;

  (*iis->func)(iis->chan, &iis->fd, NULL);
}
}

XtPointer xim_addInput(XimDataPtr xim, int fd, 
		       void (*func)(IoChan*, int*, void*), IoChan* chan)
{
  if (IISDEBUG)
    cerr << "xim_addInput() " << fd << endl;

  iis->func = func;
  iis->fd = fd;
  iis->chan = chan;
  Tcl_CreateFileHandler(fd, TCL_READABLE, (void (*)(void*,int))iisIO, NULL);

  return (void*)fd;
}

void xim_removeInput(XimDataPtr xim, int fd)
{
  if (IISDEBUG)
    cerr << "xim_removeInput() " << fd << endl;

  Tcl_DeleteFileHandler(fd);
}

void xim_initialize(XimDataPtr xim, int config, int nframes, int hardreset)
{
  // from ximtool xim_initialize

  xim->nframes = 0;
  get_fbconfig(xim);

  for (int i=0; i<MAX_FRAMES; i++)
    xim->frames[i].frameno = 0;

  FbConfigPtr cf = &xim->fb_config[config-1];
  nframes = (nframes <= 0) ? cf->nframes : nframes;
    
  xim->fb_configno = config;

  xim->df_p = &xim->frames[0];
  xim->width = cf->width;
  xim->height = cf->height;

  xim->cursor_chan = &xim->chan[0];

  if (IISDEBUG)
    cerr << "xim_initialize() " << config << ' ' 
	 << nframes << ' ' << hardreset << endl;
}

void xim_initFrame(XimDataPtr xim, int frame, int nframes, FbConfig* config,
		   char* memModel)
{
  // from ximtool xim_initFrame

  FrameBufPtr fb = &xim->frames[frame-1];
  fb->frameno = frame;
  fb->raster = frame;
  fb->zoomras = 0;
  fb->zoommap = 0;
  fb->dispmap = 0;
  fb->colormap = DEF_COLORMAP;
  fb->offset = 0.5;
  fb->scale = 1.0;
  fb->xscale = fb->yscale = 1.0;
  fb->xmag = fb->ymag = 1.0;
  fb->xcen = fb->ycen = 0.0;
  fb->label[0] = '\0';
  fb->wcsbuf[0] = '\0';

  // my stuff

  char buf[64];
  ostrstream str(buf,64);
  str << "InitFrameCmdIIS " << frame << ends;

  if (IISDEBUG)
    cerr << "xim_initFrame() " << buf << endl;

  iis->eval(buf);
}

void xim_setDisplayFrame(XimDataPtr xim, int frame)
{
  // from imtool xim_setDisplayFrame

  xim->df_p = &xim->frames[frame-1];
  FbConfig* config = &xim->fb_config[xim->fb_configno-1];
  xim->width = config->width;
  xim->height = config->height;

  // and my stuff

  char buf[64];
  ostrstream str(buf,64);
  str << "SetDisplayFrameCmdIIS " << frame << ' '
      << config->width << ' ' << config->height << ends;

  if (IISDEBUG) {
    cerr << "xim_setDisplayFrame() " << buf << endl;
  }

  iis->eval(buf);
}

void xim_setReferenceFrame(IoChanPtr chan, int frame)
{
  // from imtool xim_setDisplayFrame

  // Ignore request if channel not active.
  if (!chan->type)
    return;

  XimDataPtr xim = (XimDataPtr)chan->xim;
  int frameno = max(1, min(MAX_FRAMES, frame));
  FrameBufPtr fb = &xim->frames[frameno-1];

  // Ignore request if not a valid frame.
  if (fb->frameno > 0) {
    chan->reference_frame = frameno;
    chan->rf_p = fb;
  }

  char buf[64];
  ostrstream str(buf,64);
  str << "SetRefFrameCmdIIS " << frame << ends;

  if (IISDEBUG)
    cerr << "xim_setReferenceFrame() " << buf << endl;

  iis->eval(buf);
}

void xim_eraseFrame(XimDataPtr xim, int frame)
{
  char buf[64];
  ostrstream str(buf,64);
  str << "EraseFrameCmdIIS " << frame << ends;

  if (IISDEBUG)
    cerr << "xim_eraseFrame() " << buf << endl;

  iis->eval(buf);
}

void xim_message(XimDataPtr xim, char* message, char* imtitle)
{
  char buf[1024];
  ostrstream str(buf,1024);
  str << "MessageCmdIIS {" << message << ' ' << imtitle << '}' << ends;

  if (IISDEBUG)
    cerr << "xim_message() " << buf << endl;

  iis->eval(buf);
}

void GtWritePixels(void* gt, int frame, void* pixels, int bits,
		   int x, int y, int dx, int dy)
{
  char buf[64];
  ostrstream str(buf,64);
  str << "WritePixelsCmdIIS " << frame << ' ' << pixels << ' ' 
      << x << ' ' << y << ' ' << dx << ' ' << dy << ends;

  if (IISDEBUG)
    cerr << "GtWritePixels() " << buf << endl;

  iis->eval(buf);
}

void GtReadPixels(void* gt, int frame, void* pixels, int nbits, 
		  int x, int y, int dx, int dy)
{
  char buf[64];
  ostrstream str(buf,64);
  str << "ReadPixelsCmdIIS " << frame << ' ' << pixels << ' ' 
      << x << ' ' << y << ' ' << dx << ' ' << dy << ends;

  if (IISDEBUG)
    cerr << "GtReadPixels() " << buf << endl;

  iis->eval(buf);
}

void xim_getCursorPos(XimDataPtr xim, int* x, int* y, int* raster, int* frame)
{
  if (IISDEBUG)
    cerr << "xim_getCursorPos()" << endl;
}

void xim_cursorMode(XimDataPtr xim, int state)
{
  if (IISDEBUG)
    cerr << "xim_cursorMode() " << state << endl;
}

void xim_setCursorPos(XimDataPtr xim, int x, int y)
{
  if (IISDEBUG)
    cerr << "xim_setCursorPos()" << endl;
}

void GtQueryColormap(void* w, int a, int* first, int* nelem, int* maxelem)
{
  if (IISDEBUG)
    cerr << "GtQueryColormap()" << endl;
}

