#include <unistd.h>

#include <fcntl.h>
#include <sys/stat.h>
#include <sys/mman.h>

#include "mmap.h"

FitsMMap::FitsMMap(const char* fn)
{
  // parse the fn and options
  parse(fn);
  if (!pName_)
    return;

  // Map the file.
  int file = open(pName_, O_RDONLY);
  if (file == -1)
    return;

  struct stat info;
  if (fstat(file, &info) < 0)
    return;

  // check for empty file
  if (info.st_size == 0)
    return;

  // map it
  mapsize_ = info.st_size;
  mapdata_ = (char*)mmap(NULL, mapsize_, PROT_READ, MAP_SHARED, file, 0);

  // close the file
  close(file);

  // so far, so good
  valid_ = 1;
}

FitsMMap::~FitsMMap()
{
  munmap((caddr_t)mapdata_, mapsize_);
}

FitsFitsMMap::FitsFitsMMap(const char* fn, ScanMode mode) : FitsMMap(fn)
{
  if (valid_)
    if (mode == EXACT || pExt_ || pIndex_)
      processExact();
    else
      processRelax();
}

FitsArrMMap::FitsArrMMap(const char* fn) : FitsMMap(fn)
{
  if (valid_)
    processArray();
}

FitsMosaicMMap::FitsMosaicMMap(const char* fn) : FitsMMap(fn)
{
  if (valid_)
    processMosaic();
}
