/* Fitsy FITS Image Library.
 */

#ifndef FITSY_H
#define FITSY_H

#ifndef XFILE_H
#include <stdio.h>
typedef FILE *File;
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include <ctype.h>

#ifndef NULL
#define NULL	0
#endif

double strtod();


#define FT_BITS		    23040
#define FT_BLOCK	     2880
#define	FT_CARDLEN	       80
#define	FT_WORDLEN	    	8
#define FT_CARDS	(FT_BLOCK/FT_CARDLEN)

#define	FT_UNSIGNED	   0x1000
#define FT_BITPIX	       -1

#define	FT_FILENAME	1024
#define FT_MAXAXES	10

#define ft_fitsbloks(bytes)	((bytes+(FT_BLOCK-1))/FT_BLOCK)

/* Card memory types
 */
#define FT_MALLOC	1
#define FT_MMAP		2
#define FT_SHMAT	3
#define FT_EXTERN	4

typedef enum {
	  FT_COMMENT 	= 1
	, FT_LOGICAL
	, FT_INTEGER
	, FT_STRING
	, FT_VALUE
	, FT_REAL
	, FT_COMPLEX
} FITSType;

typedef	struct _FITSBuff {
	char		  c[FT_CARDLEN];
} FITSBuff;

typedef	FITSBuff	 *FITSCard;

extern char *FT_Comment;

#define ft_simple(fits)		( fits->basic->simple 	)
#define ft_name(fits)		( fits->basic->name  	)
#define ft_extn(fits)		( fits->basic->extn  	)
#define ft_bitpix(fits)		( fits->basic->bitpix 	)
#define ft_naxes(fits)		( fits->basic->naxes  	)
#define ft_naxis(fits, i)	( fits->basic->naxis[i-1])
#define ft_pcount(fits)		( fits->basic->pcount  	)
#define ft_gcount(fits)		( fits->basic->gcount  	)
#define ft_databytes(fits)	( fits->basic->databytes)
#define ft_datapixls(fits)	( fits->basic->datapixls)
#define ft_databloks(fits)	( fits->basic->databloks)

typedef struct _FITSBasic {
	int	 simple;
	char	*name;
	int	 extn;

	int	 bitpix;
	int	 naxes;
	int	 naxis[FT_MAXAXES];

	int	 gcount;
	int	 pcount;

	int	 databytes;
	int	 datapixls;
	int	 databloks;
} *FITSBasic;


#define ft_hasbscale(fits)      ( fits->image->has_bscale       )
#define ft_bscale(fits)         ( fits->image->dbscale          )
#define ft_hasbzero(fits)       ( fits->image->has_bzero        )
#define ft_bzero(fits)          ( fits->image->dbzero           )
#define ft_hasscaling(fits)     ( fits->image->has_bzero || fits->image->has_bscale )
#define ft_hasblank(fits)       ( fits->image->has_blank        )
#define ft_blank(fits)          ( fits->image->blank            )

typedef struct _FITSImage {
	int	  has_bscale;
	double	  dbscale;
	int	  has_bzero;
	double	  dbzero;

	int	  has_blank;
	int	  blank;
} *FITSImage;

typedef struct _FITSTCol {
	int	 type;
	int	 n;
	int	 size;
	int	 width;
	int	 offset;
	char	*format;

	int	 heap;

	char	*name;
	char	*unit;
	char	*disp;

	int	 prec;

	int	 has_scale;
	double	 scale;
	int	 has_zero;
	double	 zero;

	int	 has_blank;
	char	*ablank;
	double	 dblank;
} *FITSTCol;

typedef struct _FITSTable {
	int	  tabtype;	/* TABLE or BINTABLE 	*/
	int	  tfields;	/* # of fields		*/
	FITSTCol  col;		/* Column pointers	*/
} *FITSTable;


#define ft_cards(fits)			( (void *)fits->cards )
#define ft_data(fits)			( fits->data  )
#define ft_ncards(fits)			( fits->ncard )
#define ft_cardnth(fits, nth)		( ( nth <= fits->ncard ) 	\
						? &fits->cards[nth-1] 	\
						: NULL )

#define ft_last(fits)			ft_cardnth(fits, fits->ncard)

#define ft_byteswap()			(*(short *)"\001\000" & 0x0001)
#define ft_seek(fits)			( fits->seek )

typedef struct _FITSHead {
	FITSCard	 cards;
	FITSCard        *index;
	FITSCard         last;
	int		 acard;
	int		 ncard;

	int		 mem;
	int		 seek;
	int		 data;

	char	 	*mmap_here;
	int		 mmap_size;

	int		 sync;

	FITSBasic	 basic;
	FITSImage	 image;
	FITSTable	 table;

	struct _FITSHead*primary;
	int		 rfcount;
} *FITSHead;

typedef struct _FITSFile {
	FITSHead	 head;
	void		*data;
	int		 bitpix;
} *FITSFile;


#ifdef __cplusplus
extern "C" {
#endif

/* cardfind.c */
FITSCard ft_cardfind (FITSHead fits, FITSCard key, int add);
FITSCard ft_cardfindidx (FITSHead fits, FITSCard key, int *match);
FITSCard ft_cardfindseq (FITSHead fits, FITSCard key, int *match);
/* cardfmt.c */
FITSCard ft_cardfmt (FITSCard card, char *name, int n, FITSType type, void *value, int prec, char *comm);
FITSCard ft_cardkey (FITSCard card, char *name, int n);
FITSCard ft_cardset (FITSCard card, FITSType type, void *value, int prec, char *comm);
FITSCard ft_cardsetl (FITSCard card, int lvalue, char *comm);
FITSCard ft_cardseti (FITSCard card, int ivalue, char *comm);
FITSCard ft_cardsetr (FITSCard card, double rvalue, int prec, char *comm);
FITSCard ft_cardsets (FITSCard card, char *svalue, char *comm);
FITSCard ft_cardclr (FITSCard card, int ncards);
FITSCard ft_cardcpy (FITSCard card1, FITSCard card2);
/* cardins.c */
FITSCard ft_cardins (FITSHead fits, FITSCard card, FITSCard here);
FITSCard ft_cardapp (FITSHead fits, FITSCard card);
FITSCard ft_carddel (FITSHead fits, FITSCard card);
/* cardpar.c */
char *ft_cardpar (FITSCard card, FITSType *type, char *value, int *index, char *comm);
char *ft_cardget (FITSCard card);
int ft_cardgetl (FITSCard card);
int ft_cardgeti (FITSCard card);
double ft_cardgetr (FITSCard card);
char *ft_cardgets (FITSCard card);
/* ftimage.c */
FITSImage ft_imageloadhead (FITSHead fits);
void ft_imagestorhead (FITSHead fits, FITSImage image);
void ft_imagefree (FITSImage image);
FITSBasic ft_basicloadhead (FITSHead fits);
void ft_basicstorhead (FITSHead fits, FITSBasic basic);
void ft_basicfree (FITSBasic basic);
/* fttable.c */
FITSTable ft_tableloadhead (FITSHead fits);
void ft_tablestorhead (FITSHead fits, FITSTable table);
void ft_tablefree (FITSTable table);
/* headfile.c */
FITSHead ft_headread0 (File file, char *card1, int n);
FITSHead ft_headread (File file);
int ft_headwrite (File file, FITSHead fits);
void *ft_fileparse (File ifile, int (*callback)(), void *cdata);
void ft_dataskip (File ifile, FITSHead fits, File ofile);
int ft_dataseek (File file, FITSHead fits);
void *ft_dataread (File file, FITSHead fits, void *dataptr, int pixtype);
int ft_simpleimageread (char *filename, FITSHead *headptr, void **dataptr, void ***data2dptr, int pixtype);
int ft_simpleimagemmap (char *filename, FITSHead *headptr, void **dataptr, void ***data2dptr, int prot, int share);
int ft_munmap(FITSHead fits);
int ft_imageread (File file, FITSHead *headptr, void **dataptr, void ***data2dptr, int pixtype, char* buf, int size);
int ft_simpleimageshm (char *filename, FITSHead *headptr, void **dataptr, void ***data2dptr, int size, int flags);
int ft_simpleimagewrite (char *filename, FITSHead fits, void *data, int pixtype);
int ft_imagewrite (File file, FITSHead fits, void *data, int pixtype);
void *ft_dataswap (void *to, void *from, int n, int type);
FITSFile ft_fileread (File file, int flag, int pixtype, int *n);
void *ft_filedata (File file, FITSFile ft);
/* headfind.c */
FITSCard ft_headfind (FITSHead fits, char *name, int n, int add);
void ft_headindex (FITSHead fits);
FITSCard ft_headfindidx (FITSHead fits, char *name, int n, int *match);
FITSCard ft_headfindseq (FITSHead fits, char *name, int n, int *match);
/* headinit.c */
FITSHead ft_headinit (void *cards, int acard);
FITSHead ft_headcopy (FITSHead fits);
void ft_primlink (FITSHead prim, FITSHead fits);

void ft_headfree (FITSHead fits, int freecards);
void ft_setcards (FITSHead fits, void *cards);
/* headins.c */
FITSCard ft_headinsl (FITSHead fits, char *name, int n, int lvalue, char *comm, FITSCard here);
FITSCard ft_headinsi (FITSHead fits, char *name, int n, int ivalue, char *comm, FITSCard here);
FITSCard ft_headinsr (FITSHead fits, char *name, int n, double dvalue, int prec, char *comm, FITSCard here);
FITSCard ft_headinss (FITSHead fits, char *name, int n, char *svalue, char *comm, FITSCard here);
FITSCard ft_headinsv (FITSHead fits, char *name, int n, char *vvalue, char *comm, FITSCard here);
FITSCard ft_headappl (FITSHead fits, char *name, int n, int lvalue, char *comm);
FITSCard ft_headappi (FITSHead fits, char *name, int n, int ivalue, char *comm);
FITSCard ft_headappr (FITSHead fits, char *name, int n, double dvalue, int prec, char *comm);
FITSCard ft_headapps (FITSHead fits, char *name, int n, char *svalue, char *comm);
FITSCard ft_headappv (FITSHead fits, char *name, int n, char *vvalue, char *comm);
FITSCard ft_headdel (FITSHead fits, char *name, int n);
/* headset.c */
FITSCard ft_headsetl (FITSHead fits, char *name, int n, int lvalue, char *comm);
FITSCard ft_headseti (FITSHead fits, char *name, int n, int ivalue, char *comm);
FITSCard ft_headsetr (FITSHead fits, char *name, int n, double rvalue, int prec, char *comm);
FITSCard ft_headsets (FITSHead fits, char *name, int n, char *svalue, char *comm);
/* headget.c */
char *ft_headget (FITSHead fits, char *name, int n, FITSCard *card);
int ft_headgetl (FITSHead fits, char *name, int n, int ldefault, FITSCard *card);
int ft_headgeti (FITSHead fits, char *name, int n, int idefault, FITSCard *card);
double ft_headgetr (FITSHead fits, char *name, int n, double ddefault, FITSCard *card);
char *ft_headgets (FITSHead fits, char *name, int n, char *sdefault, FITSCard *card);

FITSHead ft_primary (FITSHead fits);
FITSHead ft_headmerge (FITSHead fits1, FITSHead fits2);
void	ft_synchead (FITSHead fits);
void	ft_syncdata (FITSHead fits);

void ft_acht (int type1, void* v1, int type2, void* v2, int npix, int direction, int hasscale, double dbscale, double dbzero);

char **ft_make2d (char *ptr, int size, int xstar, int ystar, int xsize, int ysize);
int ft_datawrite (File file, FITSHead fits, void *data, int pixtype);

/* fitshead.c */
void ft_parsefilename (char *filename, char *name, char *extn, int *indx, char *tail, int max);
File ft_fitsheadopen (char *filename, FITSHead *head, char *tail, int size);

#ifdef __cplusplus
}
#endif
#endif
