/*
 *      Copyrights:
 *
 *    	  Copyright (c) 1996, 1998 Smithsonian Astrophysical Observatory
 *
 *        Permission to use, copy, modify, distribute, and  sell  this
 *        software  and  its  documentation  for any purpose is hereby
 *        granted without  fee,  provided  that  the  above  copyright
 *        notice  appear  in  all  copies and that both that copyright
 *        notice and this permission notice appear in supporting docu-
 *        mentation,  and  that  the  name  of the  Smithsonian Astro-
 *	  physical Observatory not be used in advertising or publicity
 *  	  pertaining to distribution of the software without specific,
 *	  written  prior  permission.   The Smithsonian  Astrophysical 
 *        Observatory makes no representations about  the  suitability
 *	  of  this  software for any purpose.  It is provided  "as is"
 *	  without express or implied warranty.
 *        THE  SMITHSONIAN  ASTROPHYSICAL  OBSERVATORY  DISCLAIMS  ALL
 *	  WARRANTIES  WITH  REGARD  TO  THIS  SOFTWARE,  INCLUDING ALL
 *	  IMPLIED  WARRANTIES  OF  MERCHANTABILITY AND FITNESS, IN  NO
 *	  EVENT SHALL THE  SMITHSONIAN  ASTROPHYSICAL  OBSERVATORY  BE 
 *	  LIABLE FOR  ANY SPECIAL, INDIRECT  OR  CONSEQUENTIAL DAMAGES
 *	  OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS  OF USE,  DATA
 *	  OR  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 *	  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
 *	  THE  USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

/*
 *
 * xpa.h - include file for the X Public Access mechanism
 *
 */

#ifndef	__xpap_h
#define	__xpap_h

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#ifdef __STDC__
#include <stdlib.h>
#include <malloc.h>
#endif
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/socket.h>         /* struct sockaddr, SOCK_STREAM, ... */
#include <netinet/in.h>         /* struct in_addr, struct sockaddr_in */
#include <sys/un.h>
#include <tcp.h>
#include <word.h>
#include <find.h>

/*
 * The flag for setting non-blocking I/O varies a bit from Unix to Unix
 */
#ifdef HPUX
#  define NONBIO O_NONBLOCK
#else
#  define NONBIO O_NDELAY
#endif

/* we always want to know about errors when a read/write would block */
#ifndef EWOULDBLOCK
#define EWOULDBLOCK     EAGAIN
#endif

/* we need to close-on-exec the sockets */
#ifdef WHONEEDS
#ifndef FD_CLOEXEC
#define FD_CLOEXEC 1
#endif
#endif

/* for listen() routine */
#define XPA_MAXLISTEN 1000

/* types of xpa request */
#define XPA_SET		1
#define XPA_GET		2
#define XPA_INFO	3

/* status flags for xpa server */
#define XPA_STATUS_ACTIVE 	1
#define XPA_STATUS_FREE 	2
#define XPA_STATUS_READBUF	4
#define XPA_STATUS_ENDBUF	8

/* status flags for xpa clients */
#define XPA_CLIENT_IDLE		0
#define XPA_CLIENT_ACTIVE	1
#define XPA_CLIENT_START	2

/* server mode flags for receive, send, info */
/* crafted so that they are turned on by default */
#define XPA_MODE_BUF		1
#define XPA_MODE_FILLBUF	2
#define XPA_MODE_FREEBUF	4
#define XPA_MODE_ACL		8

/* default modes for receive, send, info */
#define XPA_DEF_MODE_REC  (XPA_MODE_BUF|XPA_MODE_FILLBUF|XPA_MODE_FREEBUF|XPA_MODE_ACL)
#define XPA_DEF_MODE_SEND (XPA_MODE_BUF|XPA_MODE_FREEBUF|XPA_MODE_ACL)
#define XPA_DEF_MODE_INFO (XPA_MODE_ACL)

/* client mode flags */
#define XPA_CLIENT_BUF		1
#define XPA_CLIENT_FD		2
#define XPA_CLIENT_ACK		4
#define XPA_CLIENT_VERIFY	8

/* prefix for XPA messages */
#define XPA_PREFIX	"XPA$"

/* error codes -- these must match the strings in xpa_error */
/* always make 0 an OK return */
#define XPA_RTN_OK	0
#define XPA_RTN_NOAUTH	1
#define XPA_RTN_NOCONN	2
#define XPA_RTN_NOHOST	3
#define XPA_RTN_NOBUF	4
#define XPA_RTN_NOCMD	5
#define XPA_RTN_NOREC	6
#define XPA_RTN_NOSEND	7
#define XPA_RTN_NOINFO	8
#define XPA_RTN_UNCMD	9
#define XPA_RTN_NOCMD2	10
#define XPA_RTN_NOTARG	11
#define XPA_RTN_NODCHAN	12
#define XPA_RTN_NODATA	13

/* connection methods */
#define XPA_INET	1
#define XPA_UNIX	2

/* select loop types */
#define XPA_XPA_LOOP	1
#define XPA_XT_LOOP	2
#define XPA_TCL_LOOP	3

/* name server management */
#define XPA_NSINET	"*:*"
#define XPA_NSIPORT	14285
#define XPA_NSUNIX	"xpans_unix"
#define XPA_RETRIES	50

/* access control */
#define XPA_ACLS	"gis"
#define XPA_ACLFILE	"$HOME/xpa.acl"
#define XPA_DEFACL	"*:* $host +"

/* misc */
#define XPA_IOSIZE     		4096
#define XPA_BIOSIZE    		204800
#define XPA_MAXIOSIZE  		2048000

/* these can be changed by user environment variable */
#define XPA_MAXHOSTS		100
#define XPA_LONG_TIMEOUT	180
#define XPA_SHORT_TIMEOUT	30
#define XPA_TMPDIR		"/tmp/.xpa"
#define XPA_VERBOSITY		1

#include <xpa.h>

#endif /* __xpa.h */
